import {getBrowserData} from '@/api/onlineDev/visualDev'

/**
 * Parse the time to string
 * @param {(Object|string|number)} time
 * @param {string} cFormat
 * @returns {string | null}
 */
export function parseTime(time, cFormat) {
  if (arguments.length === 0 || !time) {
    return null
  }
  const format = cFormat || '{y}-{m}-{d} {h}:{i}:{s}'
  let date
  if (typeof time === 'string') {
    if (/^[0-9]+$/.test(time)) {
      // support "1548221490638"
      time = parseInt(time)
    } else {
      // support safari
      // https://stackoverflow.com/questions/4310953/invalid-date-in-safari
      time = time.replace(new RegExp(/-/gm), '/')
    }
  } else {
    if (typeof time === 'string' && /^[0-9]+$/.test(time)) {
      time = parseInt(time)
    }
    if (typeof time === 'number' && time.toString().length === 10) {
      time = time * 1000
    }
    date = new Date(time)
  }
  const formatObj = {
    y: date.getFullYear(),
    m: date.getMonth() + 1,
    d: date.getDate(),
    h: date.getHours(),
    i: date.getMinutes(),
    s: date.getSeconds(),
    a: date.getDay()
  }
  const time_str = format.replace(/{([ymdhisa])+}/g, (result, key) => {
    const value = formatObj[key]
    // Note: getDay() returns 0 on Sunday
    if (key === 'a') {
      return ['日', '一', '二', '三', '四', '五', '六'][value]
    }
    return value.toString().padStart(2, '0')
  })
  return time_str
}

/**
 * @param {number} time
 * @param {string} option
 * @returns {string}
 */
export function formatTime(time, option) {
  if (('' + time).length === 10) {
    time = parseInt(time) * 1000
  } else {
    time = +time
  }
  const d = new Date(time)
  const now = Date.now()

  const diff = (now - d) / 1000

  if (diff < 30) {
    return '刚刚'
  } else if (diff < 3600) {
    // less 1 hour
    return Math.ceil(diff / 60) + '分钟前'
  } else if (diff < 3600 * 24) {
    return Math.ceil(diff / 3600) + '小时前'
  } else if (diff < 3600 * 24 * 2) {
    return '1天前'
  }
  if (option) {
    return parseTime(time, option)
  } else {
    return d.getMonth() + 1 + '月' + d.getDate() + '日' + d.getHours() + '时' + d.getMinutes() + '分'
  }
}
export function formatDate(date, fmt) {
  if (/(y+)/.test(fmt)) {
    fmt = fmt.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length))
  }
  let o = {
    'M+': date.getMonth() + 1,
    'd+': date.getDate(),
    'h+': date.getHours(),
    'm+': date.getMinutes(),
    's+': date.getSeconds()
  }
  for (let k in o) {
    if (new RegExp(`(${k})`).test(fmt)) {
      let str = o[k] + ''
      fmt = fmt.replace(RegExp.$1, RegExp.$1.length === 1 ? str : padLeftZero(str))
    }
  }
  return fmt
}

function padLeftZero(str) {
  return ('00' + str).substr(str.length)
}

/**
 * @param {string} url
 * @returns {Object}
 */
export function getQueryObject(url) {
  url = url == null ? window.location.href : url
  const search = url.substring(url.lastIndexOf('?') + 1)
  const obj = {}
  const reg = /([^?&=]+)=([^?&=]*)/g
  search.replace(reg, (rs, $1, $2) => {
    const name = decodeURIComponent($1)
    let val = decodeURIComponent($2)
    val = String(val)
    obj[name] = val
    return rs
  })
  return obj
}

/**
 * @param {string} input value
 * @returns {number} output value
 */
export function byteLength(str) {
  // returns the byte length of an utf8 string
  let s = str.length
  for (var i = str.length - 1; i >= 0; i--) {
    const code = str.charCodeAt(i)
    if (code > 0x7f && code <= 0x7ff) s++
    else if (code > 0x7ff && code <= 0xffff) s += 2
    if (code >= 0xdc00 && code <= 0xdfff) i--
  }
  return s
}

/**
 * @param {Array} actual
 * @returns {Array}
 */
export function cleanArray(actual) {
  const newArray = []
  for (let i = 0; i < actual.length; i++) {
    if (actual[i]) {
      newArray.push(actual[i])
    }
  }
  return newArray
}

/**
 * @param {Object} json
 * @returns {Array}
 */
export function param(json) {
  if (!json) return ''
  return cleanArray(
    Object.keys(json).map((key) => {
      if (json[key] === undefined) return ''
      return encodeURIComponent(key) + '=' + encodeURIComponent(json[key])
    })
  ).join('&')
}

/**
 * @param {string} url
 * @returns {Object}
 */
export function param2Obj(url) {
  const search = decodeURIComponent(url.split('?')[1]).replace(/\+/g, ' ')
  if (!search) {
    return {}
  }
  const obj = {}
  const searchArr = search.split('&')
  searchArr.forEach((v) => {
    const index = v.indexOf('=')
    if (index !== -1) {
      const name = v.substring(0, index)
      const val = v.substring(index + 1, v.length)
      obj[name] = val
    }
  })
  return obj
}

/**
 * @param {string} val
 * @returns {string}
 */
export function html2Text(val) {
  const div = document.createElement('div')
  div.innerHTML = val
  return div.textContent || div.innerText
}

/**
 * Merges two objects, giving the last one precedence
 * @param {Object} target
 * @param {(Object|Array)} source
 * @returns {Object}
 */
export function objectMerge(target, source) {
  if (typeof target !== 'object') {
    target = {}
  }
  if (Array.isArray(source)) {
    return source.slice()
  }
  Object.keys(source).forEach((property) => {
    const sourceProperty = source[property]
    if (typeof sourceProperty === 'object') {
      target[property] = objectMerge(target[property], sourceProperty)
    } else {
      target[property] = sourceProperty
    }
  })
  return target
}

/**
 * @param {HTMLElement} element
 * @param {string} className
 */
export function toggleClass(element, className) {
  if (!element || !className) {
    return
  }
  let classString = element.className
  const nameIndex = classString.indexOf(className)
  if (nameIndex === -1) {
    classString += '' + className
  } else {
    classString = classString.substr(0, nameIndex) + classString.substr(nameIndex + className.length)
  }
  element.className = classString
}

/**
 * @param {string} type
 * @returns {Date}
 */
export function getTime(type) {
  if (type === 'start') {
    return new Date().getTime() - 3600 * 1000 * 24 * 90
  } else {
    return new Date(new Date().toDateString())
  }
}

/**
 * @param {Function} func
 * @param {number} wait
 * @param {boolean} immediate
 * @return {*}
 */
export function debounce(func, wait, immediate) {
  let timeout, args, context, timestamp, result

  const later = function () {
    // 据上一次触发时间间隔
    const last = +new Date() - timestamp

    // 上次被包装函数被调用时间间隔 last 小于设定时间间隔 wait
    if (last < wait && last > 0) {
      timeout = setTimeout(later, wait - last)
    } else {
      timeout = null
      // 如果设定为immediate===true，因为开始边界已经调用过了此处无需调用
      if (!immediate) {
        result = func.apply(context, args)
        if (!timeout) context = args = null
      }
    }
  }

  return function (...args) {
    context = this
    timestamp = +new Date()
    const callNow = immediate && !timeout
    // 如果延时不存在，重新设定延时
    if (!timeout) timeout = setTimeout(later, wait)
    if (callNow) {
      result = func.apply(context, args)
      context = args = null
    }

    return result
  }
}

/**
 * This is just a simple version of deep copy
 * Has a lot of edge cases bug
 * If you want to use a perfect deep copy, use lodash's _.cloneDeep
 * @param {Object} source
 * @returns {Object}
 */
export function deepClone(source) {
  if (!source && typeof source !== 'object') {
    throw new Error('error arguments', 'deepClone')
  }
  const targetObj = source.constructor === Array ? [] : {}
  Object.keys(source).forEach((keys) => {
    if (source[keys] && typeof source[keys] === 'object') {
      targetObj[keys] = deepClone(source[keys])
    } else {
      targetObj[keys] = source[keys]
    }
  })
  return targetObj
}

/**
 * @param {Array} arr
 * @returns {Array}
 */
export function uniqueArr(arr) {
  return Array.from(new Set(arr))
}

/**
 * @returns {string}
 */
export function createUniqueString() {
  const timestamp = +new Date() + ''
  const randomNum = parseInt((1 + Math.random()) * 65536) + ''
  return (+(randomNum + timestamp)).toString(32)
}

/**
 * Check if an element has a class
 * @param {HTMLElement} elm
 * @param {string} cls
 * @returns {boolean}
 */
export function hasClass(ele, cls) {
  return !!ele.className.match(new RegExp('(\\s|^)' + cls + '(\\s|$)'))
}

/**
 * Add class to element
 * @param {HTMLElement} elm
 * @param {string} cls
 */
export function addClass(ele, cls) {
  if (!hasClass(ele, cls)) ele.className += ' ' + cls
}

/**
 * Remove class from element
 * @param {HTMLElement} elm
 * @param {string} cls
 */
export function removeClass(ele, cls) {
  if (hasClass(ele, cls)) {
    const reg = new RegExp('(\\s|^)' + cls + '(\\s|$)')
    ele.className = ele.className.replace(reg, ' ')
  }
}
export function numberChinese(ele) {
  let num = parseFloat(ele)
  let strOutput = '',
    strUnit = '仟佰拾亿仟佰拾万仟佰拾元角分'
  num += '00'
  var intPos = num.indexOf('.')
  if (intPos >= 0) {
    num = num.substring(0, intPos) + num.substr(intPos + 1, 2)
  }
  strUnit = strUnit.substr(strUnit.length - num.length)
  for (let i = 0; i < num.length; i++) {
    strOutput += '零壹贰叁肆伍陆柒捌玖'.substr(num.substr(i, 1), 1) + strUnit.substr(i, 1)
  }
  return strOutput
    .replace(/零角零分$/, '整')
    .replace(/零[仟佰拾]/g, '零')
    .replace(/零{2,}/g, '零')
    .replace(/零([亿|万])/g, '$1')
    .replace(/零+元/, '元')
    .replace(/亿零{0,3}万/, '亿')
    .replace(/^元/, '零元')
}

export function loopRelationList(fields) {
  const list = []
  const loop = (data, parent) => {
    if (!data) return
    if (data.__config__ && data.__config__.jnpfKey === 'tab' && data.__config__.children && Array.isArray(data.__config__.children)) {
      loop(data.__config__.children, data.__config__.jnpfKey)
    }
    if (Array.isArray(data) && data.length) {
      for (let i = 0; i < data.length; i++) {
        loop(data[i], parent)
      }
    }
    if (Object.prototype.toString.call(data) === '[object Object]' && parent === 'tab' && data.__config__.children && data.__config__.children.length) {
      loop(data.__config__.children, parent)
    }
    if (data.__config__ && data.__config__.jnpfKey) {
      if (data.__config__.layout === 'colFormItem' && data.__vModel__) {
        list.push(data)
      }
    }
  }
  loop(fields)
  return list
}
export function loopRelationTabelList(fields) {
  const list = []
  const loop = (data, parent) => {
    if (!data) return
    if (data.__config__ && data.__config__.jnpfKey === 'table' && data.__config__.children && Array.isArray(data.__config__.children)) {
      loop(data.__config__.children)
    }
    if (data.__config__ && data.__config__.jnpfKey === 'tab' && data.__config__.children && Array.isArray(data.__config__.children)) {
      loop(data.__config__.children, data.__config__.jnpfKey)
    }
    if (Array.isArray(data) && data.length) {
      for (let i = 0; i < data.length; i++) {
        loop(data[i], parent)
      }
    }
    if (Object.prototype.toString.call(data) === '[object Object]' && parent === 'tab' && data.__config__.children && data.__config__.children.length) {
      loop(data.__config__.children, parent)
    }
    if (data.__config__ && data.__config__.jnpfKey) {
      if (data.__config__.layout === 'colFormItem' && data.__vModel__) {
        list.push(data)
      }
    }
  }
  loop(fields)
  return list
}
// 获取当前子表单及全局变量
export function loopActiveTabelList(fields, relationName) {
  const list = []
  const loop = (data, parent) => {
    if (!data) return
    if (data.__config__ && data.__config__.jnpfKey === 'table' && data.__config__.tableName === relationName && data.__config__.children && Array.isArray(data.__config__.children)) {
      loop(data.__config__.children)
    }
    if (data.__config__ && data.__config__.jnpfKey === 'tab' && data.__config__.children && Array.isArray(data.__config__.children)) {
      loop(data.__config__.children, data.__config__.jnpfKey)
    }
    if (Array.isArray(data) && data.length) {
      for (let i = 0; i < data.length; i++) {
        loop(data[i], parent)
      }
    }
    if (Object.prototype.toString.call(data) === '[object Object]' && parent === 'tab' && data.__config__.children && data.__config__.children.length) {
      loop(data.__config__.children, parent)
    }
    if (data.__config__ && data.__config__.jnpfKey) {
      if (data.__config__.layout === 'colFormItem' && data.__vModel__) {
        list.push(data)
      }
    }
  }
  loop(fields)
  return list
}
// 自定义表达式变量
export function loopCustomTabelList(fields) {
  const list = []
  const loop = (data, fatherLabel) => {
    if (!data) return
    if (data.__config__ && data.__config__.jnpfKey === 'table' && data.__config__.children && Array.isArray(data.__config__.children)) {
      loop(data.__config__.children, data.__config__.label)
    }
    if (Array.isArray(data) && data.length) {
      for (let i = 0; i < data.length; i++) {
        loop(data[i], fatherLabel)
      }
    }
    if (data.__config__ && data.__config__.jnpfKey) {
      if (data.__config__.layout === 'colFormItem' && data.__vModel__) {
        if (fatherLabel && data.__config__.relationTable) {
          data.__config__.label = fatherLabel + '.' + data.__config__.label
          // data.__vModel__ = data.__config__.relationTable + '_formulaF_' + data.__vModel__
          data.__vModelName__ = data.__config__.relationTable + '.' + data.__vModelName__
        } else {
          // data.__vModel__ = 'formulaF_' + data.__vModel__
          // data.__vModelName__ = 'formulaF_' + data.__vModelName__
        }
        list.push(data)
      }
    }
  }
  loop(fields)
  return list
}
// 获取详情
export const detail = async (modelId, primaryTableId, idKey, value) => {
  const query = [
    {
      cubeColumnId: idKey,
      function: 'eq',
      values: [value]
    }
  ]
  const params = {
    pageQuery: {
      currentPage: 1,
      pageSize: 1
    },
    primaryTableId: primaryTableId,
    foreignTableId: [],
    modelId: modelId,
    browserRequest: idKey ? JSON.stringify(query) : JSON.stringify([])
  }
  const res = await getBrowserData(modelId, params)
  const data = res.data.data
  return {
    associationColumnDict: data.associationColumnDict,
    associationData: data.associationData,
    associationTableData: data.associationTableData,
    columnDict: data.columnDict,
    data: data.pageData.records && data.pageData.records.length ? data.pageData.records[0] : {}
  }
}

//构造搜索时的主子表结构
export function getSeachPrimaryAndForeignTable(config) {
  const formData = JSON.parse(config.formData)
  const columnData = JSON.parse(config.columnData)
  let dataForm = {...config}
  // 主表信息
  let TablePrimary = formData.fields.filter((o) => o.__config__.jnpfKey !== 'table')
  let TablePrimaryFlatten = loopRelationList(TablePrimary)
  let baseVisualdevTablePrimary = {
    formData: JSON.stringify({ fields: TablePrimaryFlatten }),
    visualdevTables: config.visualdevTables,
    dbLinkId: config.dbLinkId,
    primaryStrategy: config.primaryStrategy,
    primaryTableName: config.primaryTableName,
    deleteDataCheckReferenceRelation: formData.deleteDataCheckReferenceRelation,
    deleteDataIgnoringReferenceRelationIds: JSON.stringify(formData.deleteDataIgnoringReferenceRelationIds)
  }
  // 子表信息
  let TableForeign = formData.fields.filter((o) => o.__config__.jnpfKey === 'table')
  let baseVisualdevTableForeign = TableForeign.map((o) => {
    if (o.__config__.children && o.__config__.children.length) {
      return {
        formData: JSON.stringify({ fields: o.__config__.children }),
        visualdevTables: JSON.stringify(o.tables),
        dbLinkId: o.dbLinkId,
        oldDbLinkId: o.oldDbLinkId,
        foreignColumnId: o.relationConf.foreignModel,
        foreignColumnName: o.relationConf.foreignModelName,
        primaryColumnId: o.relationConf.primaryModel,
        primaryColumnName: o.relationConf.primaryModelName
      }
    }
  })
  // 关联信息
  let relationMapList = loopRelationList(formData.fields)
  let AssociationColumn = relationMapList.filter((o) => o.relationData && o.relationData.RelationTableCubeId)
  let PrimaryAssociationColumn = []
  if (AssociationColumn.length) {
    for (let o = 0; o < AssociationColumn.length; o++) {
      let data = {
        associationVisualdevTableId: AssociationColumn[o].relationData.itemRelationTable, //关联表单id
        associationCubeId: AssociationColumn[o].relationData.RelationTableCubeId, //关联表单数据集id
        associationColumnId: [], // 被关联表字段id集合
        visualdevTableColumnId: '', //主表字段id
        associationType: AssociationColumn[o].relationData.isCheck ? 2 : 1, //1多行关联 2多列关联
        associationPrimaryId: AssociationColumn[o].relationData.PrimaryId, // 被关联表主键id
        associationSeparator: AssociationColumn[o].relationData.associationSeparator, // 分隔符
        plaintext: '' // 明文
      }
      let isCheckFlag = AssociationColumn[o].relationData.isCheck
      if (AssociationColumn[o].relationData.itemRelationList.length) {
        let itemRList = AssociationColumn[o].relationData.itemRelationList
        if (isCheckFlag) {
          let sumbitData = deepClone(data)
          for (let z = 0; z < itemRList.length; z++) {
            if (itemRList[z].columnId && itemRList[z].drawId && itemRList[z].drawId === AssociationColumn[o].__vModel__) {
              sumbitData.associationColumnId.push(itemRList[z].columnId)
              sumbitData.plaintext = itemRList[z].plaintext
            }
          }
          sumbitData.visualdevTableColumnId = AssociationColumn[o].__vModel__
          PrimaryAssociationColumn.push(sumbitData)
        } else {
          for (let z = 0; z < itemRList.length; z++) {
            if (itemRList[z].columnId && itemRList[z].drawId) {
              let sumbitData = deepClone(data)
              sumbitData.associationColumnId.push(itemRList[z].columnId)
              sumbitData.visualdevTableColumnId = itemRList[z].drawId
              sumbitData.plaintext = itemRList[z].plaintext
              PrimaryAssociationColumn.push(sumbitData)
            }
          }
        }
      }
    }
  }
  baseVisualdevTablePrimary.associationColumnSize = PrimaryAssociationColumn.length
  baseVisualdevTablePrimary.associationTable = JSON.stringify(columnData.associationTable)
  return {
    baseVisualdev: {...dataForm, associationRelation: JSON.stringify(PrimaryAssociationColumn)},
    baseVisualdevTablePrimary,
    baseVisualdevTableForeign,
  }
}
